	TITLE	'BOOT LOADER MODULE FOR CP/M 3.0'

	; BOOT3.ASM  v.1.0  02/22/2023

	; Written by Wayne Parham  (wayne@parhamdata.com)

	; Four drive support:
	; Drives 0 and 1 are 330Kb diskettes
	; Drives 2 and 3 are 8Mb IDE drives 

TRUE		EQU	-1
FALSE		EQU	NOT TRUE

BDOS		EQU	5
CR		EQU	13
LF		EQU	10

BANKED		EQU	FALSE	;<--------------- NON-BANKED VERSION

	; Public labels:
	PUBLIC	?INIT,?LDCCP,?RLCCP,?TIME

	; External labels:
	EXTRN	?PMSG,?CONIN
	EXTRN	@CIVEC,@COVEC,@AIVEC,@AOVEC,@LOVEC
	EXTRN 	@CBNK,?BNKSL
	EXTRN	@SEC,@MIN,@HOUR,@DATE
	EXTRN	LDIRi,MEMCPY
   IF	BANKED
	EXTRN	BANKBUF		;128 byte buffer for warm boots
   ENDIF

	; INCLUDE Z-80 MACROS:
	MACLIB	Z80

   IF	BANKED
	DSEG
   ELSE
	CSEG
   ENDIF


;------------------------------------------------------------------
; System initialization
;------------------------------------------------------------------
?INIT:
	LXI	H, 8000H	;device 0
	SHLD	@CIVEC		;master console input
	SHLD	@COVEC		;and output

	LXI	H, 4000H	;device 1
	SHLD	@LOVEC		;list output

	LXI	H, 02000H	;device 2
	SHLD	@AIVEC		;aux console input
	SHLD	@AOVEC		;aux console output

	LXI	H, SIGNON$MSG
	JMP	?PMSG		;print sign-on message


;------------------------------------------------------------------
; Disk system loader
;------------------------------------------------------------------
?LDCCP:				; CCP loader
	XRA	A
	STA	CCP$FCB+15
	LXI	H, 0		;start at beginning of file
	SHLD	FCB$NR

	LXI	D, CCP$FCB	;point to FCB for CCP.COM
	CALL	OPEN		;open it
	INR	A
	JNZ	CCP$FOUND	;if found, load it

	LXI	H, CCP$MSG	;if not, print error
	CALL	?PMSG		;asking to correct problem
	CALL	?CONIN		;wait for user response
	JMP	?LDCCP		;and try again

CCP$FOUND:
	LXI	D, 0100H	;point to TPA starting address
	CALL	SETDMA		;and tell BDOS to write there
	LXI	D, 128		;set multi-sector flag
	CALL	SETMULTI	;to allow 16Kb to be read
	LXI	D, CCP$FCB	;point to FCB for CCP.com
	CALL	READ		;and read it into memory

   IF	BANKED
	LXI	H, 0100H	;point to TPA starting address
	MVI	B, 25		;set [B] to indicate 25 sectors
	LDA	@CBNK		;get current bank
	PUSH	PSW		;and save it
LD$1:
	PUSH	B		;save sector count
	MVI	A, 1		;select TPA bank
	CALL	?BNKSL
	LXI	B, 128		;transfer 128 bytes to buffer
	LXI	D, BANKBUF	;buffer address in [DE]
	PUSH	H		;save source address
	PUSH	D		;and destination
	PUSH	B		;and count
	CALL	LDIRi		;move sector to buffer
	MVI	A, 2		;put CCP in bank 2
	CALL	?BNKSL
	POP	B		;retrieve count,
	POP	H		;destination (which will be new source addr)
	POP	D		;and source (which will be new destination)
	CALL	LDIRi		;move sector data to bank 2
	XCHG			;next address will be new source address
	POP	B		;retrieve sector count
	DCR	B		;decrement sector count
	JNZ	LD$1		;and loop 'til done
	POP	PSW		;then restore original bank number
	JMP	?BNKSL		;and return afterwards
   ELSE
	RET			;non-banked return
   ENDIF


;------------------------------------------------------------------
; Warm-start system reloader
;------------------------------------------------------------------
?RLCCP:				;CCP reloader
   IF	BANKED
	LXI	H, 0100H	;point to starting address
	MVI	B, 25		;set [B] to indicate 25 sectors
	LDA	@CBNK		;get current bank
	PUSH	PSW		;and save it
RL$1:
	PUSH	B		;save sector count
	MVI	A, 2		;select bank 2
	CALL	?BNKSL
	LXI	B, 128		;transfer 128 bytes to buffer
	LXI	D, BANKBUF	;buffer address in [DE]
	PUSH	H		;save source address
	PUSH	D		;and destination
	PUSH	B		;and count
	CALL	LDIRi		;move sector to buffer
	MVI	A, 1		;put CCP in bank 1
	CALL	?BNKSL
	POP	B		;retrieve count,
	POP	H		;destination (which will be new source addr)
	POP	D		;and source (which will be new destination)
	CALL	LDIRi		;move sector data to bank 1
	XCHG			;next address will be new source address
	POP	B		;retrieve sector count
	DCR	B		;decrement sector count
	JNZ	RL$1		;and loop 'til done
	POP	PSW		;then restore original bank number
	JMP	?BNKSL		;and return afterwards
   ELSE
	JMP	?LDCCP		;non-banked jumps to CCP loader
   ENDIF			;and returns from there

?TIME:				;What time is it?	
	RET			;I dunno

   IF	BANKED			;even if banked,
	CSEG			;put everything below
   ENDIF			;in common memory


;------------------------------------------------------------------
; BDOS call definitions
;------------------------------------------------------------------
OPEN:				;Open FCB
	MVI	C, 15
	JMP	BDOS

SETDMA:				;Set data transfer address
	MVI	C, 26
	JMP	BDOS

SETMULTI:			;Set record count for multi-sector transfers
	MVI	C, 44
	JMP	BDOS

READ:				;Read records
	MVI	C, 20
	JMP	BDOS


;------------------------------------------------------------------
; Module data
;------------------------------------------------------------------

CCP$MSG:			;Loader error message
	DB	CR,LF,'BIOS ERR ON A: NO CCP.COM FILE',0

CCP$FCB:			;FCB for CCP.COM
	DB	1		;drive A
	DB	'CCP     COM'	;filename (8.3)
	DB	0,0,0,0
	DS	16
FCB$NR:	DB	0,0,0

SIGNON$MSG:			;Bootstrap message
	db	CR,LF,LF,'CP/M Plus v3.0hd',CR,LF,LF,LF,0

	END
